package cn.com.infosec.crypto.signers;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DSA;
import cn.com.infosec.crypto.params.ECKeyParameters;
import cn.com.infosec.crypto.params.ECPrivateKeyParameters;
import cn.com.infosec.crypto.params.ECPublicKeyParameters;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.math.ec.ECAlgorithms;
import cn.com.infosec.math.ec.ECConstants;
import java.math.BigInteger;
import java.security.SecureRandom;

public class ECGOST3410Signer implements DSA {
   ECKeyParameters key;
   SecureRandom random;

   public void init(boolean forSigning, CipherParameters param) {
      if (forSigning) {
         if (param instanceof ParametersWithRandom) {
            ECGOST3410Signer var10000 = this;
            ParametersWithRandom var3;
            this.random = (var3 = (ParametersWithRandom)param).getRandom();
            var10000.key = (ECPrivateKeyParameters)var3.getParameters();
         } else {
            this.random = new SecureRandom();
            this.key = (ECPrivateKeyParameters)param;
         }
      } else {
         this.key = (ECPublicKeyParameters)param;
      }

   }

   public BigInteger[] generateSignature(byte[] message) {
      int var2;
      byte[] var3 = new byte[var2 = message.length];

      for(int var4 = 0; var4 != var2; ++var4) {
         var3[var4] = message[var2 - 1 - var4];
      }

      BigInteger var5;
      var5 = new BigInteger.<init>(1, var3);
      BigInteger var6 = this.key.getParameters().getN();

      while(true) {
         BigInteger var10000;
         BigInteger message1;
         BigInteger var8;
         do {
            var10000 = message1 = new BigInteger;
            var2 = var6.bitLength();
            var10000.<init>(var2, this.random);
         } while(var10000.equals(var8 = ECConstants.ZERO));

         BigInteger var9;
         if (!(var9 = this.key.getParameters().getG().multiply(message1).getX().toBigInteger().mod(var6)).equals(var8)) {
            var10000 = message1;
            message1 = ((ECPrivateKeyParameters)this.key).getD();
            if (!(message1 = var10000.multiply(var5).add(message1.multiply(var9)).mod(var6)).equals(var8)) {
               return new BigInteger[]{var9, message1};
            }
         }
      }
   }

   public boolean verifySignature(byte[] message, BigInteger r, BigInteger s) {
      int var4;
      byte[] var5 = new byte[var4 = message.length];

      for(int var6 = 0; var6 != var4; ++var6) {
         var5[var6] = message[var4 - 1 - var6];
      }

      BigInteger message1;
      message1 = new BigInteger.<init>(1, var5);
      BigInteger var8 = this.key.getParameters().getN();
      BigInteger var9;
      if (r.compareTo(var9 = ECConstants.ONE) >= 0 && r.compareTo(var8) < 0) {
         if (s.compareTo(var9) >= 0 && s.compareTo(var8) < 0) {
            s = s.multiply(message1 = message1.modInverse(var8)).mod(var8);
            message1 = var8.subtract(r).multiply(message1).mod(var8);
            return ECAlgorithms.sumOfTwoMultiplies(this.key.getParameters().getG(), s, ((ECPublicKeyParameters)this.key).getQ(), message1).getX().toBigInteger().mod(var8).equals(r);
         } else {
            return false;
         }
      } else {
         return false;
      }
   }
}
